#ifndef _LINUX_JOYSTICK_H
#define _LINUX_JOYSTICK_H

/*
 * $Id: joystick.h,v 1.7 2002/01/20 17:54:44 vojtech Exp $
 *
 *  Copyright (C) 1996-2000 Vojtech Pavlik
 *
 *  Sponsored by SuSE
 *
 *  Updated to be more cross-platform, by Philip Brown, Jan 2002
 */

/*
 * This header file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Should you need to contact me, the author, you can do so either by e-mail -
 * mail your message to <vojtech@ucw.cz>, or by paper mail: Vojtech Pavlik,
 * Simunkova 1594, Prague 8, 182 00 Czech Republic
 */

#ifdef __linux__
# include <asm/types.h>
# include <linux/input.h>
  /* the "intXX_t" types are a standard. So make the linux
   * code look more like the standard, rather than the other way around
   */
# define int16_t __s16
# define int32_t __s32
# define uint8_t __u8
# define uint16_t __u16
# define uint32_t __u32
#else
# include <sys/ioccom.h>
# include <sys/ioctl.h>
# include <sys/types.h>

#ifndef _IOC_READ
/* This section cribbed from 
 * http://www.linuxtv.org/cgi-bin/cvsweb.cgi/dietlibc/include/sys/sparc-ioctl.h
 */
# define _IOC_NONE        1U
# define _IOC_READ        2U
# define _IOC_WRITE       4U

# define _IOC_NRBITS      8
# define _IOC_TYPEBITS    8
# define _IOC_SIZEBITS    8
# define _IOC_RESVBITS    5
# define _IOC_DIRBITS     3

# define _IOC_NRMASK      ((1 << _IOC_NRBITS)-1)
# define _IOC_TYPEMASK    ((1 << _IOC_TYPEBITS)-1)
# define _IOC_RESVMASK    ((1 << _IOC_RESVBITS)-1)
# define _IOC_SIZEMASK    ((1 << _IOC_SIZEBITS)-1)
# define _IOC_DIRMASK     ((1 << _IOC_DIRBITS)-1)

# define _IOC_NRSHIFT     0
# define _IOC_TYPESHIFT   (_IOC_NRSHIFT + _IOC_NRBITS)
# define _IOC_SIZESHIFT   (_IOC_TYPESHIFT + _IOC_TYPEBITS)
# define _IOC_RESVSHIFT   (_IOC_SIZESHIFT + _IOC_SIZEBITS)
# define _IOC_DIRSHIFT    (_IOC_RESVSHIFT + _IOC_RESVBITS)


# define _IOC(dir,type,nr,size) \
        (((dir)  << _IOC_DIRSHIFT) | \
         ((type) << _IOC_TYPESHIFT) | \
         ((nr)   << _IOC_NRSHIFT) | \
         ((size) << _IOC_SIZESHIFT))
#endif
#endif



/*
 * Version
 */

#define JS_VERSION		0x020100

/*
 * Types and constants for reading from /dev/js
 */

#define JS_EVENT_BUTTON		0x01	/* button pressed/released */
#define JS_EVENT_AXIS		0x02	/* joystick moved */
#define JS_EVENT_INIT		0x80	/* initial state of device */

struct js_event {
	uint32_t time;	/* event timestamp in milliseconds */
	int16_t value;	/* value */
	uint8_t type;	/* event type */
	uint8_t number;	/* axis/button number */
};

/*
 * IOCTL commands for joystick driver
 */

#define JSIOCGVERSION		_IOR('j', 0x01, uint32_t)				/* get driver version */

#define JSIOCGAXES		_IOR('j', 0x11, uint8_t)				/* get number of axes */
#define JSIOCGBUTTONS		_IOR('j', 0x12, uint8_t)				/* get number of buttons */
#define JSIOCGNAME(len)		_IOC(_IOC_READ, 'j', 0x13, len)			/* get identifier string */

#define JSIOCSCORR		_IOW('j', 0x21, struct js_corr)			/* set correction values */
#define JSIOCGCORR		_IOR('j', 0x22, struct js_corr)			/* get correction values */

#define JSIOCSAXMAP		_IOW('j', 0x31, uint8_t[ABS_MAX])			/* set axis mapping */
#define JSIOCGAXMAP		_IOR('j', 0x32, uint8_t[ABS_MAX])			/* get axis mapping */
#define JSIOCSBTNMAP		_IOW('j', 0x33, uint16_t[KEY_MAX - BTN_MISC])	/* set button mapping */
#define JSIOCGBTNMAP		_IOR('j', 0x34, uint16_t[KEY_MAX - BTN_MISC])	/* get button mapping */

/*
 * Types and constants for get/set correction
 */

#define JS_CORR_NONE		0x00	/* returns raw values */
#define JS_CORR_BROKEN		0x01	/* broken line */

struct js_corr {
	int32_t coef[8];
	int16_t prec;
	uint16_t type;
};

/***********************************************************************\
 * v0.x compatibility definitions.                                     *
\***********************************************************************/


#define JS_RETURN		sizeof(struct JS_DATA_TYPE)
#define JS_TRUE			1
#define JS_FALSE		0
#define JS_X_0			0x01
#define JS_Y_0			0x02
#define JS_X_1			0x04
#define JS_Y_1			0x08
#define JS_MAX			2

#define JS_DEF_TIMEOUT		0x1300
#define JS_DEF_CORR		0
#define JS_DEF_TIMELIMIT	10L

#define JS_SET_CAL		1
#define JS_GET_CAL		2
#define JS_SET_TIMEOUT		3
#define JS_GET_TIMEOUT		4
#define JS_SET_TIMELIMIT	5
#define JS_GET_TIMELIMIT	6
#define JS_GET_ALL		7
#define JS_SET_ALL		8

struct JS_DATA_TYPE {
	int buttons;
	int x;
	int y;
};

struct JS_DATA_SAVE_TYPE {
	int JS_TIMEOUT;
	int BUSY;
	long JS_EXPIRETIME;
	long JS_TIMELIMIT;
	struct JS_DATA_TYPE JS_SAVE;
	struct JS_DATA_TYPE JS_CORR;
};

#endif /* _LINUX_JOYSTICK_H */
